{{
************************************
* iRobot Create Serial Interface  v2.0 for ServerBot
* by Joe Lucia - 2joester@gmail.com
* http://irobotcreate.googlepages.com
* This object requires two (2) cogs, one for the serial port
* and one for the CreateCog to process the serial port data
*
* The HM55B is also using another Cog at the moment.
************************************
}}

CON
  '' * Create 25-pin Cargo Bay Connector Pins for referen
  ' 1 - RXD
  ' 2 - TXD 
  ' 3 - Power Control Toggle
  ' 4 - Analog Input
  ' 5 - Digital input 1
  ' 6 - Digital input 3
  ' 7 - Digital output 1
  ' 8 - Switched 5V (100ma)
  ' 9 - Vpwr (unregulated, 500ma)
  ' 10,11,12 - Switched Vpwr (1500ma)
  ' 13 - Robot charging
  ' 14,16,21,25 - GND
  ' 15 - BRC (digital input, baud-rate-change to 19200)
  ' 17 - Digital input 0
  ' 18 - Digital input 2
  ' 19 - Digital output 0
  ' 20 - Digital output 2
  ' 22 - Low side driver 0 (500ma)
  ' 23 - Low side driver 1 (500ma, IR LED)
  ' 24 - Low side driver 2 (1500ma)

  DebugRXPin                    = 31
  DebugTXPin                    = 30                        

  '' * oi.h - Definitions for the Open Interface 
  ' Command values
  CmdSoftReset    =7
  CmdStart        =128
  CmdBaud         =129
  CmdControl      =130
  CmdSafe         =131
  CmdFull         =132
  CmdSpot         =134
  CmdClean        =135
  CmdDemo         =136
  CmdDrive        =137
  CmdMotors       =138
  CmdLeds         =139
  CmdSong         =140
  CmdPlay         =141
  CmdSensors      =142
  CmdDock         =143
  CmdPWMMotors    =144
  CmdDriveWheels  =145
  CmdOutputs      =147
  CmdSensorStream =148
  CmdSensorList   =149
  CmdStreamControl=150
  CmdIRChar       =151
   
  ' Sensor byte indices - offsets in packets 0, 5 and 6
  SenBumpDrop     =0            
  SenWall         =1
  SenCliffL       =2
  SenCliffFL      =3
  SenCliffFR      =4
  SenCliffR       =5
  SenVWall        =6
  SenOverC        =7
  SenIRChar       =10
  SenButton       =11
  SenDist1        =12
  SenDist0        =13
  SenAng1         =14
  SenAng0         =15
  SenChargeState  =16
  SenVolt1        =17
  SenVolt0        =18
  SenCurr1        =19
  SenCurr0        =20
  SenTemp         =21
  SenCharge1      =22
  SenCharge0      =23
  SenCap1         =24
  SenCap0         =25
  SenWallSig1     =26
  SenWallSig0     =27
  SenCliffLSig1   =28
  SenCliffLSig0   =29
  SenCliffFLSig1  =30
  SenCliffFLSig0  =31
  SenCliffFRSig1  =32
  SenCliffFRSig0  =33
  SenCliffRSig1   =34
  SenCliffRSig0   =35
  SenInputs       =36
  SenAInput1      =37
  SenAInput0      =38
  SenChAvailable  =39
  SenOIMode       =40
  SenOISong       =41
  SenOISongPlay   =42
  SenStreamPckts  =43
  SenVel1         =44
  SenVel0         =45
  SenRad1         =46
  SenRad0         =47
  SenVelR1        =48
  SenVelR0        =49
  SenVelL1        =50
  SenVelL0        =51
   
  ' Sensor packet sizes
  Sen0Size        =26
  Sen1Size        =10
  Sen2Size        =6
  Sen3Size        =10
  Sen4Size        =14
  Sen5Size        =12
  Sen6Size        =52
   
  ' Sensor bit masks
  WheelDropFront  =$10
  WheelDropLeft   =$08
  WheelDropRight  =$04
  BumpLeft        =$02
  BumpRight       =$01
  BumpBoth        =$03
  BumpEither      =$03
  WheelDropAll    =$1C
  ButtonAdvance   =$04
  ButtonPlay      =$01
  
  LeftWheelOverC  =$10
  RightWheelOverC =$08
  LD2OverC        =$04
  LD0OverC        =$02
  LD1OverC        =$01
   
  ' LED Bit Masks
  LEDAdvance      =$08
  LEDPlay         =$02
  LEDsBoth        =$0A
   
  ' Baud codes
  Baud300         =0
  Baud600         =1
  Baud1200        =2
  Baud2400        =3
  Baud4800        =4
  Baud9600        =5
  Baud14400       =6
  Baud19200       =7
  Baud28800       =8
  Baud38400       =9
  Baud57600       =10
  Baud115200      =11
   
  ' Drive radius special cases
  RadStraight     =32768
  RadCCW          =1
  RadCW           =-1
   
  ' Baud UBRRx values
  Ubrr300         =3839
  Ubrr600         =1919
  Ubrr1200        =959
  Ubrr2400        =479
  Ubrr4800        =239
  Ubrr9600        =119
  Ubrr14400       =79
  Ubrr19200       =59
  Ubrr28800       =39
  Ubrr38400       =29
  Ubrr57600       =19
  Ubrr115200      =9
     
  ' Charge states.  Goes into WAIT_CHG before TRKL_CHG
  #0, NO_CHG, RECON_CHG, FULL_CHG, TRKL_CHG, WAIT_CHG, CHG_FAULT
   
  '' Create OI Modes
  #0, MODE_UNKNOWN, MODE_PASSIVE, MODE_SAFE, MODE_FULL, MODE_OFF

  ' IR Commands from Remotes
  IRSendAll     = 128
  IRLeft        = 129
  IRForward     = 130
  IRRight       = 131
  IRSpot        = 132
  IRMax         = 133
  IRClean       = 136
  IRPause       = 137
  IRPower       = 138
  IRDock        = 143

  ' Home Base IR Codes (240..254)
  HBDetect      = 240  ' %1111_0000
  HBForceField  = 242  ' %1111_0010
  HBGreen       = 244  ' %1111_0100
  HBGreenForce  = 246  ' %1111_0110
  HBRed         = 248  ' %1111_1000
  HBRedForce    = 250  ' %1111_1010
  HBRedGreen    = 252  ' %1111_1100
  HBRedGreenForce= 254 ' %1111_1110

  ' IR value returned when nothing is seen
  IRNothing     = 255

  
  ' Chargers
  WALL_CHARGER    =1
  DOCK_CHARGER    =2
  
  DEFAULTPOLLINTERVAL           = 100                    ' ms
  TURNSPEED                     = 50                   ' optimal turn speed
  TURNERROR                     = 8                     ' +/- degrees

  '' COMPASS MODULE
  hm55Data1                     = 13                    ' port 13
  hm55Enable1                   = 14                    ' port 14
  hm55Clock1                    = 15                    ' port 15
  COMPASSPOLLINTERVAL           = 100                   ' ms
  HEADINGCORRECTINTERVAL        = 150
  USECOMPASS                    = true                 ' uses compass instead of create sensor readings to determine current Bearing

  DRIVINGTURNRADIUS             = 900  

  #0, CC_IDLE, CC_DRIVE, CC_DRIVEDISTANCE, CC_TURNLEFT, CC_TURNRIGHT, CC_TURNTODEG, CC_TURNDEG, CC_SETOIMODE, CC_PLAYSONG, CC_DOCK, CC_UNDOCK, CC_SENDIRCHAR, CC_SPEEDADJUST, CC_SPEEDSET

OBJ
  SerialCreate  :  "FullDuplexSerial"
  SerialDebug   :  "FullDuplexSerial"
  math          :  "DynamicMathLib"
  clock         :  "Clock"
  HM55B_1       : "HM55B Compass Module Asm"            ' 1 cog
  f             : "FloatMath"
  
VAR
  '' cog Variables
  long  CreateStack[1024]        ' stack for CreateCog()
  byte  cog                     ' CreateCog() id
  long  isPaused                ' when set, Speed, Distance, and Heading will not be processed
  
  '' Create to Propeller Pins
  byte  rxPin                   ' Propeller RXpin for Create's TXpin                   
  byte  txPin                   ' Propeller TXpin for Create's RXpin
  byte  RobotPwrTogglePin       ' Create Pin 3  (Power control toggle low-to-high) OUTPUT
  byte  RobotPowerSensePin      ' Create Pin 8  (Switched 5V 100ma) INPUT 10k resistor
  byte  RobotChargePin          ' Create Pin 13 (Robot charging) INPUT 10k resistor
  byte  i2cSDAPin, i2cSCLPin

  '' Sensor Buffer
  long  isSensorData            ' set to TRUE after requesting a sensor packet
  byte  sensors[Sen6Size]       ' array of sensor packet bytes
  byte  sensorsPtr              ' pointer to current sensor packet byte in sensors[] array
  long  _SensorPacketReq        ' increments every time we Request a packet from the Create
  long  _SensorPacketID         ' increments each time we Get a packet from the Create
  long  LastPacketClk           ' last time we received a sensor packet
  long  _Enabled                ' if not SET the we won't request a packet from the Create (is FALSE when Create Power is OFF)
  long  ReadInterval            ' interval (ms) to request a Packet from the Create

  '' Sensor Data
  byte  _Bumpers                ' left,middle,right bumpers
  byte  _VirtualWall            ' virtual wall detected
  byte  _IRChar                 ' IR Char received
  byte  _OverCurrent            ' over-current sensor 0..31
  byte  _OverCurrentCounter     ' helps ignore short-term over-current, increments each time we get a packet with OverCurrent set
  byte  _Buttons                ' create Button status
  byte  _ChargeState            ' current charging state
  byte  _ChargerAvailable       ' charger is available
  byte  _OIMode                 ' current OI mode reported by the Create                                                                      
  byte  _CliffL, _CliffFL, _CliffFR, _CliffR           ' cliff sensors (0 or 1)
  long  _CliffLv, _CliffFLv, _CliffFRv, _CliffRv       ' cliff sensor values
  long  _BatteryVoltage         ' current voltage of battery mV
  long  _BatteryUsage           ' current ma usage
  long  _BatteryRemaining       ' current mah remaining
  long  _BatteryCapacity        ' total mah battery starts with
  long  _BatteryTemperature     ' temperature of battery
  long  _WallPresent            ' true if wall is available
  long  _Wall                   ' right wall sensor value approx 0..500
  long  _Velocity               ' current speed as reported via sensor packet
  long  _LeftWheelVelocity
  long  _RightWheelVelocity
  long  _Radius                 ' current turning radius as reported via sensor packet 

  '' Calculated sensor data
  long  _mmTravelled            ' generic running total of millimeters travelled, can be reset without affecting distance/angle measurements
  long  _isUndocking            ' TRUE if we are attempting to Undock, get set to false as soon as we determine we are no longer docked
  long  _isDocking              ' TRUE when we are attempting to Dock
  long  _Bearing                ' current robot heading based on Angle sensor data
  long  _Distance               ' accumulated Distance Traveled
  long  _Angle                  ' accumulated Angle
  long  _XCoord, _YCoord        ' current x,y coordinates

  '' Target Distance, Angle, and Heading settings
  long  _TargetXCoord
  long  _TargetYCoord           ' coordinates of where we want to be
  long  _TargetSpeed
  long  _TargetAngle            ' direction of target
  long  _TargetDistance         ' distance of target
  long  GoToDistance            ' we are activelty working toward TargetDistance
  long  _TargetHeading          ' desired robot Heading
  long  _TargetBearing          ' direction to go to get to Target X,Y Coordinates
  long  _TargetRange            ' distance to Target X,Y Coordinates
  long  GoToTargetXY            ' we are actively working toward TargetXCoord, TargetYCoord

  '' Status variables
  long  PwrFlag                 ' tracks last-kinow power state to detect the changes
  long  AutoDock
  long  AutoUndock              ' Undock when done charging
  long  _lastSpeed              ' last REQUESTED speed, set in Drive() and DriveDirect() routines
  long  _lastTurnDirection      ' indicates we are currently turning left -1, not 0, or right 1
  long  _lastTurnSpeed          ' speed at which we are turning, If we are turning
  long  _lastMovement           ' We need to move every so often to put a load on the battery for an accurate reading
  long  LastTurnClk             ' last CNT that we made a turn adjustment
  long  LastSpeedUpdateClk      ' last time we adjusted the speed toward our Target speed

  '' Command Queue
  long  lastCurrentCommandID    ' id of last command we handled
  long  CurrentCommandID        ' id of current command
  long  CurrentCommand          ' drive process currently being executed
  long  CurrentCommandParams[5]

  '' Compass Variables
  long  lastTheta
  long  lastThetaClk

  '' Debugging
  long _DebugRXPin, _DebugTXPin                              

{{ Create Routines ****************************************************************** }}
PUB Start(_rxpin, _txpin, _pwrtogglepin, _pwrsenpin, _chrgsenpin) : ok
  stop
  
  {Create pins}
  rxpin:=_rxpin
  txpin:=_txpin

  RobotPwrTogglePin := _pwrtogglepin
  RobotPowerSensePin := _pwrsenpin
  RobotChargePin := _chrgsenpin
  _DebugRXPin:=_DebugTXPin := -1

  ok := (cog := cognew(CreateCog, @CreateStack)+1)

PUB Stop
  if cog
    SerialCreate.stop
    SerialDebug.Stop
    cogstop(cog~ - 1)

PRI Debug(drxpin, dtxpin)
  _DebugRXPin:=drxpin
  _DebugTXPin:=dtxpin
  SerialDebug.Start(_DebugRXPin, _DebugTXPin, 0, 57600)

PRI CurrentMS
  return clock.TimeInMilliseconds

PRI TimeDiff(tim)
  return clock.TimeInMilliseconds - tim
    
PRI CreateCog | x
  {{ This cog will process the sensor data from the Create and RFID serial ports
      as well as poll the sonar and compass i2c sensors }}
      
  ReadInterval := DEFAULTPOLLINTERVAL                ' interval at which to poll the Create for sensor data

  dira[RobotPowerSensePin]~     'input
  dira[RobotPwrTogglePin]~~     'output
  dira[RobotChargePin]~         'input

  outa[RobotPwrTogglePin]~

  clock.start

  'math.Start
  '' Initialize Create Serial Interface
  SerialCreate.Start(rxpin, txpin, 0, 57600)
  '' Initialize Compass Object
  HM55B_1.start(hm55Enable1,hm55Clock1,hm55Data1)

  'Debug(debugrxpin, debugtxpin)
  
  LastPacketClk := CurrentMS
  LastThetaClk := CurrentMS
  LastSpeedUpdateClk := CurrentMS
  LastTurnClk := CurrentMS
  _TargetHeading:=-1
  
  AutoUndock:=false
  AutoDock:=false
  ClearTarget  
  if not IsPowerOn
    PowerOn                                               ' power-up
  if isDocked
    PwrFlag:=true
  else
    PwrFlag:=not IsPowerOn                                ' handle transition initially
  _Enabled:=true

  repeat
    doIdleLoop

PRI doIdleLoop                                          '' Get and Process sensor data
    'Clock.Update
    
    GetSensorData

    CheckPower

    'CheckCommand

    if not isPaused

      CheckSpeed

      CheckDistance

      CheckHeading


PRI tx(c)
  SerialCreate.tx(c)
  if _DebugRXPin => 0
    SerialDebug.tx(c)
      
CON '' Request, Buffer, and Extract Sensor data from Create
PRI ClearSensorData
  bytefill(@sensors, 0, Sen6Size)
  SerialCreate.rxflush
  isSensorData~

PRI GetSensorData | rxchar                              '' Get sensor data from Create serial interface
  repeat while (rxChar:=SerialCreate.rxcheck)=>0
    if isSensorData
      ' we are receiving sensor data
      sensors[sensorsPtr++]:=rxChar
      if sensorsPtr=>Sen6Size
        ' we have a complete sensor packet now
        isSensorData~
        ProcessSensorData
        _SensorPacketID++

  if _Enabled          ' request a sensor packet
    if TimeDiff(LastPacketClk) > ReadInterval 
        RequestSensorPacket(6)

  if TimeDiff(lastThetaClk) > COMPASSPOLLINTERVAL
    lastTheta := HM55B_1.theta 'ompassValue
    lastThetaClk := CurrentMS
    {
    case ((lastTheta*1000/8170)*359/1000)
      43..46: _Bearing:=0
      109..111: _Bearing:=90
      159..162: _Bearing:=180
      219..222: _Bearing:=270 
    }
    
PRI RequestSensorPacket(_packetid)                      '' Send a request for a packet to the Create
  ClearSensorData
  sensorsPtr:=0                                         ' reset buffer position
  isSensorData~~
  SerialCreate.rxflush
  serialcreate.tx(CmdSensors)
  serialcreate.tx(_packetid)
  _SensorPacketReq++
  LastPacketClk:=CurrentMS  
  'return 1

PRI ProcessSensorData | dx, ax, v                          '' Process a complete sensor packet
  ' this is called every time we receive a complete sensor packet

  _BatteryVoltage:=sensors[SenVolt1]<<8 | sensors[SenVolt0]
  _BatteryUsage:=sensors[SenCurr1]<<8 | sensors[SenCurr0]
  _BatteryUsage:=~~_BatteryUsage
  _BatteryRemaining:=sensors[SenCharge1]<<8 | sensors[SenCharge0]
  _BatteryCapacity:=sensors[SenCap1]<<8 | sensors[SenCap0]
  _BatteryRemaining:=~~_BatteryRemaining

  _Bumpers:=sensors[SenBumpDrop]
  _WallPresent:=sensors[SenWall]
  _Wall:=sensors[SenWallSig1]<<8 | sensors[SenWallSig0]
  _VirtualWall:=sensors[SenVWall]

  _IRChar:=sensors[SenIRChar]   

  dx:=sensors[SenDist1]<<8 | sensors[SenDist0]
  dx:=~~dx
  _Distance+=dx
  if GoToDistance and (_TargetDistance<>0)
    _TargetDistance-=dx
    if _TargetDistance < 0
      _TargetDistance := 0

  _mmTravelled+=dx
                   
  ax:=sensors[SenAng1]<<8 | sensors[SenAng0]
  ax:=~~ax
  _Angle-=ax

  _Bearing-=ax
  if _Bearing>359
    _Bearing-=360
  elseif _Bearing<0
    _Bearing+=360

  CalcNewPosition(dx, Bearing)

  _OverCurrent:=sensors[SenOverC]
  if _OverCurrent
    _OverCurrentCounter++
  else
    _OverCurrentCounter~
  _Buttons:=sensors[SenButton]
  _ChargeState:=sensors[SenChargeState]
  _ChargerAvailable:=sensors[SenChAvailable]
  _BatteryTemperature:=sensors[SenTemp]

  _CliffL:=sensors[SenCliffL]
  _CliffFL:=sensors[SenCliffFL]
  _CliffFR:=sensors[SenCliffFR]
  _CliffR:=sensors[SenCliffR]

  _CliffLv := sensors[SenCliffLSig1]<<8 | sensors[SenCliffLSig0]
  _CliffFLv := sensors[SenCliffFLSig1]<<8 | sensors[SenCliffFLSig0]
  _CliffFRv := sensors[SenCliffFRSig1]<<8 | sensors[SenCliffFRSig0]
  _CliffRv := sensors[SenCliffRSig1]<<8 | sensors[SenCliffRSig0]

  _OIMode:=sensors[SenOIMode]

  v := sensors[SenVel1]<<8 | sensors[SenVel0]
  _Velocity := ~~v
  v := sensors[SenVelL1]<<8 | sensors[SenVelL0]
  _LeftWheelVelocity := ~~v
  v := sensors[SenVelR1]<<8 | sensors[SenVelR0]
  _RightWheelVelocity := ~~v
  v := sensors[SenRad1]<<8 | sensors[SenRad0]
  _Radius := ~~v

  'if _Velocity <> 0
  '  _lastMovement := clock.SecondsR

PRI DEG2PROP(Deg)               'Convert Deg to 13-bit Propeller angle for lookup
    Result := (Deg * 1024)/45
 
PRI Cos(angl)                  'Cos angle is 13-bit ; Returns a 16-bit signed value
    Result := sin(angl + $800)
 
PRI Sin(angl)                  'Sin angle is 13-bit ; Returns a 16-bit signed value
    Result := angl << 1 & $FFE
    if angl & $800
       Result := word[$F000 - Result]
    else
       Result := word[$E000 + Result]
    if angl & $1000
       -Result
       
PRI CalcNewPosition(dist, angl) | x, y, h               '' Calculate new position using dist and angl
  ' update current x,y coordinats
  ' radians = angl/57.296= angl/(180/pi)
  ' newx = dist * sin(radians)
  ' newy = dist * cos(radians)
  if dist <> 0
    _XCoord += ( sin( deg2prop(angl)) * dist / 65536)
    _YCoord += ( cos( deg2prop(angl)) * dist / 65536)
    {
    _TargetBearing:=(atan2(_XCoord-_TargetXCoord, _YCoord-_TargetYCoord)*1000) / 17.4533
    _TargetBearing+=180
    if(_TargetBearing>359)
      _TargetBearing-=360
    }

CON '' Checks for new Command, Power changes, Distance, Speed, and Heading conditions
PRI CheckCommand  '' Check CurrentCommandID to see if we should handle a new function
  if lastCurrentCommandID == CurrentCommandID
    return

  '' Set appropriate variables to carry-out a new Command

  case CurrentCommand
    CC_IDLE:  '' stop and sit, don't adjust heading
      doDriveStop

    CC_DRIVE: '' drive a particular speed and angle
      doDrive(CurrentCommandParams[0], CurrentCommandParams[1])      
              
    CC_DRIVEDISTANCE:  '' drive a particular speed and angle for specific distance, then go CC_IDLE
      doDriveDistance(CurrentCommandParams[0], CurrentCommandParams[1])
      
    CC_TURNLEFT:
    CC_TURNRIGHT:

    CC_TURNTODEG:
      doSetHeading(CurrentCommandParams[0])

    CC_TURNDEG:
      doAdjustHeading(CurrentCommandParams[0])

    CC_SETOIMODE:
      case CurrentCommandParams[0]
        CmdStart: doPassiveMode
        CmdSafe: doSafeMode
        'CmdFull: doFullMode
        CmdSoftReset: doSoftReset      
                
    CC_PLAYSONG:
      doPlaySong(CurrentCommandParams[0])
    CC_DOCK:
      doDock
    CC_UNDOCK:
      doUndock
    CC_SENDIRCHAR:
      doSendIRChar(CurrentCommandParams[0])
    CC_SPEEDADJUST:
      doAdjustSpeed(CurrentCommandParams[0])
    CC_SPEEDSET:
      doSetSpeed(CurrentCommandParams[0])

  lastCurrentCommandID := CurrentCommandID

PRI CheckPower                                          '' Check power-related data 
  if not (isDocked or isDocking or isUndocking)                       '' We are not NOT docked or actively docking
    if PwrFlag   ' Power was on
      if not IsPowerOn ' now it's off
        _Enabled:=false
        PwrFlag:=false
        isSensorData~
        return
    else ' Power was off
      if IsPowerOn  ' now it's ON
        PwrFlag:=true
        _Enabled:=true
        isSensorData~
        LastPacketClk := currentMS

  '' No power, nothing else matters
  if not isPowerOn  
    return

  '' We are now docked after actively docking
  if isDocking and isDocked
    _isDocking:=false
    ClearNavData
    ClearTarget
    DelayMs(5000)
    doPassiveMode
    return

  '' UnDock when we are charged
  if isDocked and (not isUndocking) and ((_ChargeState==TRKL_CHG) or (_ChargeState==WAIT_CHG))
    '' we are docked and charging is complete
    if AutoUndock 
      UnDock
    else
      ' change mode to stop charging
      if _OIMode <> MODE_FULL
        doFullMode
        _OIMode := MODE_FULL
        _ChargeState := NO_CHG

    return

  '' Check Battery Level
  if (BatteryVoltage < 15000) ' we have a low battery
    if not (isDocking or isUndocking or (_ChargeState<>NO_CHG)) ' not charging
      if (BatteryVoltage < 14000) and AutoDock and (not isDocked)                    ' not docked
        '' check for Docking station beacon
        if (_IRChar<255) and (_IRChar>239)              ' we see the dock
          Dock                                          ' so dock
          return
      elseif isDocked                                   ' already docked
        if (_OIMode => MODE_PASSIVE)
          doSoftReset                                     ' soft reset to start charging
        return
    
PRI CheckHeading | turndir                                        '' make adjustments if Bearing doesn't match Target Bearing
  if isDocked or isDocking or isUndocking
    return
    
  if GoToTargetXY
    ' calculate new _TargetHeading

  if _TargetHeading < 0
    _LastTurnDirection := 0
    return    

  if TimeDiff(LastTurnClk) < HEADINGCORRECTINTERVAL
    return

  turndir := calcTurnDirection(_TargetHeading, TURNERROR)       ' check to see if we need to turn
  
  if turndir == 0
    if _lastspeed <> 0
      doDrive(_lastspeed, RadStraight)
    else
      doDriveStop
  elseif turndir == RadCW
    if _lastSpeed==0
      doDrive(TURNSPEED, turndir)
    else
      doDrive(_lastSpeed, -DRIVINGTURNRADIUS)
  else
    if _lastSpeed==0
      doDrive(TURNSPEED, turndir)
    else
      doDrive(_lastSpeed, DRIVINGTURNRADIUS)                            ' turn in appropriate direction
   
  _LastTurnDirection := turndir

  LastTurnClk:=CurrentMs

PRI CheckDistance | d                                      '' stop if we've reached our target distance
  if not GoToDistance
    return

  if (_LastSpeed > 0 and _TargetDistance < 30) or (_LastSpeed < 0 and _TargetDistance > -30)
    _TargetDistance:=0
    _TargetSpeed:=0
    GoToDistance := false
    doDriveStop
    if isUndocking
      _isUndocking:=false
      ClearNavData
      ClearTarget
  elseif (_TargetDistance < 450) and (_TargetSpeed > 50)
    _TargetSpeed:=50
    LastSpeedUpdateClk:=CurrentMs
    
PRI CheckSpeed
  if _OIMode<MODE_SAFE
    return
  
  if (_lastSpeed==0) and (_TargetSpeed>0)
    doSetSpeed(5)
  else
    if _TargetSpeed == _lastSpeed
      return
     
 if TimeDiff(LastSpeedUpdateClk) > 170
    if _velocity > 0
      _lastSpeed:=_velocity

    if _TargetSpeed > _lastSpeed
      doAdjustSpeed(15)
      if _lastSpeed > _TargetSpeed
        _lastSpeed:=_TargetSpeed
        doSetSpeed(_lastSpeed)
    else
      doAdjustSpeed(-15)
      if _lastSpeed < _TargetSpeed
        _lastSpeed:=_TargetSpeed
        doSetSpeed(_lastSpeed)
    LastSpeedUpdateClk:=CurrentMs
      
CON '' Methods to control the Create Power 
PRI TogglePowerPin
  dira[RobotPwrTogglePin]~~     'output

  outa[RobotPwrTogglePin]~  
  outa[RobotPwrTogglePin]~~
  outa[RobotPwrTogglePin]~  

PRI PowerToggle
  if IsPowerOn
    PowerOff
  else
    PowerOn

PRI doSoftReset
  ClearSensorData
  ClearNavData
  ClearTarget
  tx(CmdSoftReset)
  _OIMode := MODE_UNKNOWN
  DelayMs(3500)
  doPassiveMode

PUB IsPowerOn
  dira[RobotPowerSensePin]~   'input

  return ina[RobotPowerSensePin]
    
PUB PowerOn
  if not IsPowerOn
    PwrFlag:=false
    TogglePowerPin
    _OIMode := MODE_UNKNOWN
    DelayMs(3500)
    ClearSensorData
    PassiveMode

PUB PowerOff
  if IsPowerOn
    PwrFlag:=true
    TogglePowerPin
    _OIMode := MODE_UNKNOWN
    DelayMs(500)
    ClearSensorData
   
PUB SoftReset
  'AddCommand(CC_SETOIMODE, CmdSoftReset,0,0,0,0)
  doSoftReset
  
CON '' Methods to control the Create Mode 
PRI SetBaud
  SerialCreate.Start(rxpin, txpin, 0, 57600)
  DelayMs(500)
  tx(CmdStart)
  DelayMs(200)
  tx(CmdBaud)
  tx(Baud19200)
  DelayMs(200)
  SerialCreate.Start(rxpin, txpin, 0, 19200)
  doSafeMode
       
PRI doFullMode
  if _OIMode==MODE_FULL
    return
  doPassiveMode
  tx(CmdFull)
  DelayMs(2000)
  'DefineSongs
  doSetLEDs(0, 1, 0, 255)                                 ' Play LED and Power LED ON (Green)
  SerialCreate.rxflush

PRI doSafeMode
  doPassiveMode
  tx(CmdSafe)
  DelayMs(2000)
  'DefineSongs
  doSetLEDs(0, 1, 0, 0)                                   ' Play LED ON
  SerialCreate.rxflush

PRI doPassiveMode
  tx(CmdStart)
  DelayMs(500)
  SerialCreate.rxflush

PUB PassiveMode
  doPassiveMode
  'AddCommand(CC_SETOIMODE, CmdStart,0,0,0,0)

PUB FullMode
  doFullMode
  'AddCommand(CC_SETOIMODE, CmdFull,0,0,0,0)

PUB SafeMode
  doSafeMode
  'AddCommand(CC_SETOIMODE, CmdSafe,0,0,0,0)

CON '' Navigation commands 
PUB ClearCoordinates
  _XCoord:=0
  _YCoord:=0

PUB ClearNavData
  _Bearing:=0
  _Angle:=0
  _Distance:=0
  _TargetDistance:=0
  GoToDistance:=false
  GoToTargetXY:=false
  
PUB ClearTarget   '' clear target values for x,y and distance and angle and heading
  _TargetXCoord:=0
  _TargetYCoord:=0
  _TargetAngle := -1
  _TargetDistance := 0
  GoToDistance := false
  GoToTargetXY := false
  _TargetHeading := -1
  _TargetBearing := -1
  _TargetRange := 0 

PRI doDrive(velocity, radius)
  if isDocked or isdocking
    if not isUndocking
      return
  
  tx(CmdDrive)
  tx(~~velocity >> 8)                         
  tx(velocity)
  tx(~~radius >> 8)                           
  tx(radius)

PUB Drive(velocity, radius)
  doDrive(velocity, radius)
  'AddCommand(CC_DRIVE, velocity, radius, 0, 0, 0)
  
PRI doDriveDirect(RightVelocity, LeftVelocity)
  if isDocked or isDocking
    if not isUndocking
      return

  tx(CmdDriveWheels)
  tx(~~RightVelocity >> 8)
  tx(RightVelocity)
  tx(~~LeftVelocity >> 8)
  tx(LeftVelocity)

PUB LastSpeed
  if isPaused
    return 0
  else
    return _lastSpeed

'' Speed  
PRI doAdjustSpeed(mmpersec)
  if (isDocked and not isUndocking) or isDocking
    return
  'if _TargetBearing<0
  '  _TargetBearing:=Bearing
  '' add/subtract mmpersec from current Speed
  _lastSpeed+=mmpersec
  if _lastSpeed < -500
    _lastSpeed := -500
  if _lastSpeed > 500
    _lastSpeed := 500
'  if (_LastTurnDirection<>0)
'    return
  doDrive(_lastSpeed, RadStraight)

PUB AdjustSpeed(mmpersec)
  doAdjustSpeed(mmpersec)
  'AddCommand(CC_SPEEDADJUST, mmpersec, 0,0,0,0)
  
PUB AdjustTargetSpeed(mmpersec)
  if (isDocked and not isUndocking) or isDocking
    return

  
  if (mmpersec=>0) and (_TargetHeading < 0)
    _TargetHeading := Bearing    
  _TargetSpeed+=mmpersec
  if _TargetSpeed < -500
    _TargetSpeed:=-500
  if _TargetSpeed > 500
    _TargetSpeed:=500

PRI doSetSpeed(mmpersec)
  '' set speed to mmpersec
  if _lastSpeed==mmpersec
    if mmpersec<>0
      return
    
  _lastSpeed:=0
  doAdjustSpeed(mmpersec)

PUB SetSpeed(mmpersec)
  doSetSpeed(mmpersec)
  'AddCommand(CC_SPEEDSET, mmpersec, 0,0,0,0)

PUB SetTargetSpeed(mmpersec)
  _TargetSpeed:=mmpersec
{  if _lastSpeed==0
    SetSpeed(5)+}

PRI doDriveSTOP
  _isDocking:=false
  _isUndocking:=false
  doSetSpeed(0)
  ClearTarget
  _LastTurnDirection:=99
  _TargetSpeed:=0
  _TargetHeading:=-1

PUB DriveStop
  doDriveStop
  'AddCommand(CC_IDLE, 0, 0, 0, 0, 0)
  
PUB CalcTurnDirection(dheading, dPlusMinus) : turndir | diff1
  '' calculate direction to turn from current Bearing to dheading
  ' 1=left RadCCW, 0=straight, -1=right RadCW
  if(dheading>bearing)
    dheading-=360
   
  diff1:=bearing-dheading
   
  if(diff1<dPlusMinus)
    turndir:=0
  elseif(diff1=<180)
    turndir:=RadCCW
  else               
    turndir:=RadCW   
   
  return turndir

PRI FixTargetHeading
  if _TargetHeading>359
    _TargetHeading-=360
  elseif _TargetHeading<0
    _TargetHeading+=360
  
PRI doAdjustHeading(deg_offset)
  if isDocked or isDocking or isUndocking
    return
  
  if _TargetHeading<0
    _TargetHeading:=Bearing   
  
  _TargetHeading+=deg_offset
  FixTargetHeading
  _lastturndirection := 99                              ' force update

PUB AdjustHeading(deg_offset)
  doAdjustHeading(deg_offset)
  'AddCommand(CC_TURNDEG, deg_offset,0,0,0,0)

PRI doSetHeading(deg)
  if isDocked or isDocking or isUndocking
    return
  _TargetHeading:=deg
  FixTargetHeading
  LastTurnClk:=0
  _lastturndirection := 99

PUB SetHeading(deg)
  doSetHeading(deg)
  'AddCommand(CC_TURNTODEG, deg,0,0,0,0)
  
PRI doDriveDistancemm(dist, spd)                          '' Drive Distance in millimeters
  GoToDistance:=true
  _TargetDistance := dist
  if _TargetHeading<0
    _TargetHeading:=Bearing
  _lastTurnDirection:=99
  _TargetSpeed:=spd

'' FIX not starting when drivedistance is called
  if (_lastSpeed ==0)
    if dist > 0
      doSetSpeed(5) ' start out slowwwwww
    else
      doSetSpeed(-5)

PRI doDriveDistance(dist, spd)                            '' Drive Distance in Inches
  doDriveDistancemm(dist*254/10, spd)                          ' convert to millimeters

PUB DriveDistance(dist, spd)
  doDriveDistance(dist, spd)
  'AddCommand(CC_DRIVEDISTANCE, dist, spd, 0,0,0)
    
PRI doDock
  DriveStop  
  _isDocking:=true
  _isUndocking:=false
  doRunDemo(1)
  _OIMode:=0

PUB Dock
  doDock
  'AddCommand(CC_DOCK,0,0,0,0,0)

PUB UnDock
  doUnDock
  'AddCommand(CC_UNDOCK,0,0,0,0,0)
  
PRI doUnDock
  if (not isDocked) or (isUndocking) or (isDocking)
    return
  _isUndocking:=true
  doSafeMode
  ClearTarget
  doDrive(-300, RadStraight)
  DelayMs(2000)
  doDriveStop

PUB GoToXY(x,y)
  _TargetXCoord := x
  _TargetYCoord := y
  GoToTargetXY := true

PUB DrivePause
  'doDrive(0, RadStraight)
  'AddCommand(CC_DRIVE, 0, 0, 0,0,0)
  isPaused:=true

PUB DriveResume
  isPaused:=false

PUB GetCurrentCommand
  return CurrentCommand

PUB AddCommand(Cmd, Param1, Param2, Param3, Param4, Param5)
  CurrentCommand := Cmd
  CurrentCommandParams[0] := Param1    
  CurrentCommandParams[1] := Param2    
  CurrentCommandParams[2] := Param3    
  CurrentCommandParams[3] := Param4    
  CurrentCommandParams[4] := Param5    
  CurrentCommandID++
  CheckCommand

CON '' Make sensor data available to other objects 
PUB psensors(id)                                   '' returns a raw sensor data byte
  return sensors[id]

PUB CommandID
  return CurrentCommandID

PUB SensorBearing                                       '' Bearing calculated from Sensor Data
  return _Bearing

PUB Bearing                                            '' Current Heading
  if USECOMPASS
    return Compass
  else
    return _Bearing

PUB Distancemm                                            '' Total distance travelled since last Nav Reset
  return _Distance

PUB Distance | x                                              '' Distance travelled in Inches
  x := (_Distance*10/254)
  return x 

PUB Angle                                               
  return _Angle

PUB XCoordmm                                            '' X coordinates in millimeters
  return _XCoord

PUB XCoord | x                                              '' X coordinate in Inches
  x := _XCoord*10/254
  return x                               

PUB YCoordmm                                            '' Y coordinate in Millimeters
  return _YCoord

PUB YCoord | x                                            '' Y coordinate in Inches
  x := _YCoord*10/254 
  return x                              
  
PUB BatteryVoltage
  return _BatteryVoltage

PUB BatteryUsage
  return _BatteryUsage

PUB BatteryRemaining
  return _BatteryRemaining

PUB BatteryPct                                          '' returns percentage of battery remaining
  if _BatteryRemaining <> 0
    return (_BatteryRemaining * 100)/(_BatteryCapacity)
  else
    return 999

PUB BatteryCapacity
  return _BatteryCapacity

PUB BatteryTemperatureC                                 '' Battery Temperature in Degrees C
  return _BatteryTemperature

PUB BatteryTemperature                                  '' Battery Temperature in Degrees F
  return _BatteryTemperature * 9 / 5 + 32
  
PUB Bumpers
  return _Bumpers

PUB WallPresent
  return _WallPresent

PUB Wall
  return _Wall

PUB VirtualWall
  return _VirtualWall

PUB IRChar
  return _IRChar

PUB OverCurrentCounter
  return _OverCurrentCounter

PUB Buttons
  return _Buttons

PUB ChargeState
  return _ChargeState

PUB OIMode
  return _OIMode

PUB ChargerAvailable
  return _ChargerAvailable

PUB isDocked
  dira[RobotChargePin]~                                 'input
  return ina[RobotChargePin]

PUB SensorPacketID
  return _SensorPacketID
  ' _SensorPacketID := 0

PUB SensorPacketReq
  result := _SensorPacketReq
  _SensorPacketReq := 0

PUB CliffSensors                                        ' return bitmask of cliff sensors
  return (_CliffL<<3 | _CliffFL<<2 | _CliffFR<<1 | _CliffR)

PUB GetCliffSensors(sptr)                                       ' return actual values of cliff sensors
  ' pass in a pointer to 4 consecutive Words for results
  WORD[sptr] := _CliffLv
  WORD[sptr][1] := _CliffFLv
  WORD[sptr][2] := _CliffFRv
  WORD[sptr][3] := _CliffRv

PUB IsStuck
  return (_OverCurrentCounter>10)

PUB ResetOverCurrent
  _OverCurrentCounter~

PUB isDocking
  return _isDocking

PUB isUndocking
  return _isUndocking

PUB mmTravelled
  return _mmTravelled

PUB TargetHeading
  return _TargetHeading

PUB TurnDirection
  return _LastTurnDirection

PUB TargetBearing
  return _TargetBearing

PUB TargetDistance
  return (_TargetDistance*10/254)

PUB TargetSpeed
  return _TargetSpeed

PUB Compass
  return (lastTheta*1000/8170)*360/1000

CON '' More Create Commands
PUB Reset_mmTravelled
  _mmTravelled:=0

PRI DefineSongs | x
  tx(cmdSong)
  repeat x from 0 to 5
    tx(SONG0[x])
  tx(cmdSong)
  repeat x from 0 to 3
    tx(SONG1[x])
  tx(cmdSong)
  repeat x from 0 to 5
    tx(SONG2[x])

PRI doPlaySong(num)
  tx(cmdPlay)
  tx(num)

PUB PlaySong(num)
  doPlaySong(num)
  'AddCommand(CC_PLAYSONG, num, 0, 0,0,0)

PRI doSetLSD(lsd0pwm, lsd1pwm, lsd2pwm)
  tx(cmdPWMMotors) 
  tx(lsd2pwm) 
  tx(lsd1pwm) 
  tx(lsd0pwm) 

PRI doSetLEDs(advanceled, playled, powercolor, powerintensity) | b
  b := (advanceled << 3) | (playled << 1)
  tx(CmdLeds)
  tx(b)
  tx(powercolor)
  tx(powerintensity)

PRI doRunDemo(dnum)                                                               '' run a built-in demo
  tx(CmdDemo)
  tx(dnum)

PRI doSendIRChar(ch)
  tx(CmdIRChar)
  tx(ch)

PUB SendIRChar(ch)
  doSendIRChar(ch)
  'AddCommand(CC_SENDIRCHAR, ch, 0,0,0,0)

CON '' Delay Routines
PRI DelayMs(ms) | m
  waitcnt(clkfreq/1000*ms+cnt)
{
  m := clock.MillisecondsR
  repeat while clock.millisecondsr-m < ms
    'GetSensorData
}

CON '' Compass Routines
PRI CompassValue | x, y, p, status
{
    outa[NotEna..DinDout] := %100
    dira[NotEna..DinDout] := %110
    Toggle(NotEna)                                     ' make sure disabled briefly
    shiftout(DinDout, Clock, RESET, 4)
    Toggle(NotEna)                                     ' make sure disabled briefly
    shiftout(DinDout, Clock, MEASURE, 4)
    status := 0
    repeat until status == READY
      Toggle(NotEna)                                   ' briefly disable, then reenable
      shiftout(DinDout, Clock, REPORT, 4)
      status := shiftin(DinDout, Clock, 4)
    x := shiftin(DinDout, Clock, XYBits) << XYShft ~> XYShft
    y := shiftin(DinDout, Clock, XYBits) << XYShft ~> XYShft
    outa[NotEna] := 1                                  ' leave compass disabled
    return Atan2(-y,x)
}

PRI Atan2(y,x) | xs,ys,t,i,s,txx
  ys := y ~> 31
  xs := x ~> 31
  y := f.FFloat(|| y)
  x := f.FFloat(|| x)
  if y == 0.0
    if xs
      return 180
    return 0
  if x == 0.0
    if ys
      return 270
    return 90
  t := f.FDiv(y,x)
  t := f.FDiv(f.FSub(t,1.0),f.FAdd(t,1.0))
  txx := f.FMul(t,t)
  s := long[@atanTbl][0]
  repeat i from 1 to 5
    s := f.FAdd(f.FMul(s,txx),long[@atanTbl])
  s := f.FAdd(f.FMul(s,t),constant(pi/4.0))
  s := f.FRound(f.FMul(s,constant(180.0/pi)))
  if ys
    if xs
      s := 180 + s
    s := 360 - s
  if xs
    s := 180 - s
  return (s + 1) & $1FE


PRI ShiftIn(Dpin, Cpin, Bits) : Value | InBit
    dira[Dpin]~                                        ' Set data pin to input
    outa[Cpin]~                                        ' Set clock low 
    dira[Cpin]~~                                       ' Set clock pin to output 
    Value:=0                                                      
    REPEAT Bits                                        ' for number of bits                    
      outa[Cpin]~~                                     ' cycle clock                         
      outa[Cpin]~                                     
      InBit:= ina[Dpin]                                ' get bit value                          
      Value := (Value << 1) + InBit                    ' Add to value shifted by position                                         
      waitcnt(1000 + cnt)                              ' time delay

PRI ShiftOut(Dpin, Cpin, Value, Bits)
    outa[Dpin]~                                        ' Data pin = 0
    dira[Dpin]~~                                       ' Set data as output
    outa[Cpin]~
    dira[Cpin]~~                                       ' Set clock as output
    REPEAT Bits                                                                
      outa[Dpin] := Value >> (bits-1)                  ' Set output           
      Value := Value << 1                              ' Shift value right    
      outa[Cpin]~~                                     ' cycle clock          
      outa[Cpin]~                                                             
      waitcnt(1000 + cnt)                              ' delay                
    dira[Dpin]~                                        ' Set data as input

DAT
atanTbl   long   -0.0117212, 0.05265332, -0.011643287
          long   0.19354346, -0.33262348, 0.99997723
   
SONG0                   byte    0, 2, 96, 16, 84, 16   ' 6 bytes
SONG1                   byte    1, 1, 96, 8
SONG2                   byte    2, 2, 96, 8, 96, 8