'' ******************************************************************************
'' * A/D Converter with PIC 16F690                                              *
'' * Version 1.0                                                                *
'' ******************************************************************************
''
'' this object provides the PUBLIC functions:
''  -> Init  - sets up the address and inits sub-objects such
''  -> Start - try a re-start.
''  -> geti2cError - return the i2cobject error
''  -> isStarted - return the start status


VAR
  long  started

OBJ
  i2cObject   : "i2cObject"

PUB Init(_i2cSDAPin, _i2cSCLPin) : OK
  if started:=i2cObject.init(_i2cSDAPin, _i2cSCLPin, 0)
    started:=i2cObject.Start

  return started  

PUB devicePresent(vdev) : okay
  okay:=false
  ' check if device is present before allowing initialization
  if started == true
    okay := i2cObject.devicePresent(A2D_ADDRESSES[vdev])
  return okay

PUB stop
  ' stop the object
  if started == true
    started := false
  i2cObject.Stop

PUB isStarted : result
  return started

PUB geti2cError : errorCode
  return i2cObject.getError
  
PUB Read(vdev, vmem) : result
  ' return the last range
  if started == true
    i2cObject.i2cStart
    i2cObject.i2cWrite(A2D_ADDRESSES[vdev] | 0,8)
    i2cObject.i2cWrite(vmem,8)
    i2cObject.i2cWrite(2,8)
    i2cObject.i2cStart
    i2cObject.i2cWrite(A2D_ADDRESSES[vdev] | 1,8)
    result := i2cObject.i2cRead(i2cObject#_i2cACK)
    result <<= 8
    result := result + i2cObject.i2cRead(i2cObject#_i2cNAK)
    i2cObject.i2cStop

PUB ReadValues(vdev, ptr_irvalues) | x
  ' pass in a pointer to an array of 16 bytes (8 WORD values for 8 A2D sensors)
  if started == true
    i2cObject.i2cStart
    i2cObject.i2cWrite(A2D_ADDRESSES[vdev] | 0,8)
    i2cObject.i2cWrite(0,8)
    i2cObject.i2cWrite(16,8)
    i2cObject.i2cStart
    i2cObject.i2cWrite(A2D_ADDRESSES[vdev] | 1,8)
    repeat x from 0 to 14
      BYTE[ptr_irvalues][x] := i2cObject.i2cRead(i2cObject#_i2cACK)
    BYTE[ptr_irvalues][15] := i2cObject.i2cRead(i2cObject#_i2cNAK)
    i2cObject.i2cStop
     

DAT

A2D_ADDRESSES         byte    $A2